#
# This is the server logic of a Shiny web application. You can run the 
# application by clicking 'Run App' above.
#
# Find out more about building applications with Shiny here:
# 
#    http://shiny.rstudio.com/
#

library(shiny)
library(DT)
library(rgdal)
library(leaflet)
library(plotly)

carto<-readOGR("data","cartogram")
carto@data<-carto@data["gss"]
str(carto@data)

shp<-readOGR("data","constituencies_new")
shp@data<-shp@data["gss"]
str(shp@data)

# Define server logic required to draw a histogram
shinyServer(function(input, output) {
  
  combined<-read.csv("data/combined.csv")
  d<-combined[c(1:12,23)]
  d$non_voters<-d$total_voters-d$total_votes
  nms<-names(combined[,3:10])
  f<-function(x)(nms[which(x==max(x))])
  d$Win<-apply(d[,3:9],1,f)
  original<-d
  
 
  
  vote_table<-function(d)
    {
    total_votes<-sum(d$Con+d$Green+d$Lab+d$LibDem+d$UKIP+d$Plaid)
    total_voters<-sum( d$total_voters)
    Lab_per<-round(sum(d$Lab)/total_votes*100,0)
    Con_per<-round(sum(d$Con)/total_votes*100,0)
    Lab_tot<-round(sum(d$Lab)/1000000,1)
    Con_tot<-round(sum(d$Con)/1000000,1)
    turnout<-round(total_votes/total_voters*100,0)
    ddd<-data.frame(Con_per,Lab_per,Con_tot,Lab_tot,turnout)
    output$out2<-renderPrint(ddd)
    
    f<-function(x)(nms[which(x==max(x))])
    out<-table(apply(d[,3:9],1,f))
    output$out<-renderPrint(out)
    out
  }

  output$map1<-renderLeaflet({
    shp2<-merge(shp,d,by.x="gss",by.y="id")
    shp2<-shp2[!is.na(shp2$Win),]
    cls<-colorFactor(c('blue', 'green', 'red', 'orange', 'pink', 'purple'),domain = c("Con","Lab","Green","LibDem","Plaid","UKIP"))
    mp<-leaflet(shp2)%>%
      addTiles() %>%
      addPolygons(fillColor = cls(shp2$Win),color=cls(shp2$Win),label=shp2$name)
    mp
  })
  
  combined$turnout<-combined$total_votes/combined$total_voters*100
  combined$percentover40<- combined$p_older*100
  output$fig1<-renderPlotly({
    library(ggplot2)
    g0<-ggplot(combined,aes(y=turnout,x=percentover40,label=name))
    g1<-g0+geom_point(aes(colour=Win))+ ylab("Turnout of voting age population")+ xlab("Percent of voting age population over 40") + scale_colour_manual(  values = c("blue","green","red","orange","orange","purple"))+geom_smooth()
    g1<-g1 +labs(title="Turnout as function of constituency demographics")
   
    ggplotly(g1)
  })
  
  output$cartomap<-renderLeaflet({
    shp2<-merge(carto,d,by.x="gss",by.y="id")
    shp2<-shp2[!is.na(shp2$Win),]
    cls<-colorFactor(c('blue', 'green', 'red', 'orange', 'pink', 'purple'),domain = c("Con","Lab","Green","LibDem","Plaid","UKIP"))
    mp2<<-leaflet(shp2)%>%
      addTiles() %>%
      addPolygons(fillColor = cls(shp2$Win),color=cls(shp2$Win),label=shp2$name)
    mp2
  })
  
 obs<-observe(
   {
     lab2con<-input$Lab2con/100
     con2lab<-input$Con2lab/100
     
     ukip2con<-input$Ukip2con/100
     ukip2lab<-input$Ukip2lab/100
     
     con2libdem<-input$Con2libdem/100
     lab2libdem<-input$Lab2libdem/100
     
     nv2lab<-input$NV2lab/100
     nv2con<-input$NV2Con/100
     
     d$Lab<-d$Lab + d$Con*con2lab - d$Lab*lab2con + d$UKIP * ukip2lab + d$non_voters*nv2lab -d$Lab*lab2libdem
     d$Con<-d$Con + d$Lab * lab2con - d$Con * con2lab +d$UKIP * ukip2con -d$Con*con2libdem + d$non_voters*nv2con
     d$UKIP<- d$UKIP-ukip2con*d$UKIP -d$UKIP*ukip2lab
     
     d$LibDem<-d$LibDem+ d$Con*con2libdem +d$Lab*lab2libdem
     
     f<-function(x)(nms[which(x==max(x))])
     d$Previous<-d$Win
     d$Win<-apply(d[,3:9],1,f)

    
    dnew<-vote_table(d)
    output$tab1<-renderDataTable(data.frame(vote_table(d)))
    
    output$cartomap2<-renderLeaflet({
      shp2<-merge(carto,d,by.x="gss",by.y="id")
      shp2<-shp2[!is.na(shp2$Win),]
      cls<-colorFactor(c('blue', 'green', 'red', 'orange', 'pink', 'purple'),domain = c("Con","Lab","Green","LibDem","Plaid","UKIP"))
      mp<-leaflet(shp2)%>%
        addTiles() %>%
        addPolygons(data=shp2,fillColor = cls(shp2$Win),color=cls(shp2$Win),label=shp2$name)
      mp
    })
    
    
    
    dnew<-d[d$Win!=original$Win,]
    output$out3<-renderDataTable(dnew)
    
    
    output$map<-renderLeaflet({
      shp2<-merge(shp,dnew,by.x="gss",by.y="id")
      shp2<-shp2[!is.na(shp2$Win),]
      cls<-colorFactor(c('blue', 'green', 'red', 'orange', 'pink', 'purple'),domain = c("Con","Lab","Green","LibDem","Plaid","UKIP"))
      mp<-leaflet(shp2)%>%
        addTiles() %>%
        addPolygons(fillColor = cls(shp2$Win),color=cls(shp2$Win),label=shp2$name,popup=shp2$Previous)
      mp
    })
    
   }
)

})
